//
//  Shader.fsh
//  FY_Graphics2D_Test
//
//  Created by David Lareau on 2013-02-26.
//  Copyright (c) 2013 David Lareau. All rights reserved.
//

#ifdef FY_OPENGL_ES_2
precision highp float;
#endif

// in: uniform from app
uniform sampler2D activeTexture;
// in: from vertex shader
varying vec4 x_tintColor;
varying vec2 x_texture;
varying float x_opacity;
varying vec4 x_color;
varying float x_useColor;
varying float x_useGreyScale;
varying vec4 x_overlayColor;

#define BlendOverlayf(base, blend) (base < 0.5 ? (2.0 * base * blend) : (1.0 - 2.0 * (1.0 - base) * (1.0 - blend)))

void applyOpacityToColor(inout vec4 inoutColor, in float inOpacity) {
  inoutColor = inoutColor * inOpacity;
}

void premultiplyAndApplyTintOnColor(inout vec4 inoutColor, in vec4 inTintColor)
{
  inoutColor.rgb = (inTintColor.rgb * inoutColor.a * inTintColor.a + inoutColor.rgb * (1.0 - inTintColor.a));
}

void greyScaleOnColor(inout vec4 inoutColor) {
  float greyScale = (inoutColor.r + inoutColor.g + inoutColor.b) / 3.0;
  inoutColor = vec4(greyScale, greyScale, greyScale, inoutColor.a);
}

void overlayOnColor(inout vec4 inoutColor, in vec4 inOverLayColor)
{
  if (inOverLayColor.a > 0.0)
  {
    inoutColor.r = BlendOverlayf(inoutColor.r, inOverLayColor.r);
    inoutColor.g = BlendOverlayf(inoutColor.g, inOverLayColor.g);
    inoutColor.b = BlendOverlayf(inoutColor.b, inOverLayColor.b);
    inoutColor.a = BlendOverlayf(inoutColor.a, inOverLayColor.a);
  }
}

void main(void) {
  vec4 color = x_color;
  color.rgb *= color.a; // premultiply alpha

  premultiplyAndApplyTintOnColor(color, x_tintColor);
  applyOpacityToColor(color, x_opacity);
  overlayOnColor(color, x_overlayColor);
  
  if (x_useGreyScale > 0.5) {
    greyScaleOnColor(color);
  }
  gl_FragColor = color;
}